
#include <windows.h>  
#include <stdio.h>
#include "dscrparm.h"
#include "sqlext.h"  


#if defined (WIN32)
	#define IS_WIN32 TRUE
#else
	#define IS_WIN32 FALSE
#endif

#define IS_NT      IS_WIN32 && (BOOL)(GetVersion() < 0x80000000)
#define IS_WIN32S  IS_WIN32 && (BOOL)(!(IS_NT) && (LOBYTE(LOWORD(GetVersion()))<4))
#define IS_WIN95   (BOOL)(!(IS_NT) && !(IS_WIN32S)) && IS_WIN32

HINSTANCE hInst;   // current instance

LPCTSTR lpszAppName = "MyApp";
LPCTSTR lpszTitle   = "SQLDescribeParam()"; 


BOOL RegisterWin95( CONST WNDCLASS* lpwc );
VOID BuildOutputStr( LPSTR pszLBStr, SWORD fSqlType, UDWORD cbColDef, 
                                     SWORD ibScale,  SWORD  fNullable );


int APIENTRY WinMain( HINSTANCE hInstance, HINSTANCE hPrevInstance,
                      LPTSTR lpCmdLine, int nCmdShow)
{
   MSG      msg;
   HWND     hWnd; 
   WNDCLASS wc;

   wc.style         = CS_HREDRAW | CS_VREDRAW;
   wc.lpfnWndProc   = (WNDPROC)WndProc;       
   wc.cbClsExtra    = 0;                      
   wc.cbWndExtra    = 0;                      
   wc.hInstance     = hInstance;              
   wc.hIcon         = LoadIcon (hInstance, lpszAppName); 
   wc.hCursor       = LoadCursor(NULL, IDC_ARROW);
   wc.hbrBackground = (HBRUSH)(COLOR_WINDOW+1);
   wc.lpszMenuName  = lpszAppName;              
   wc.lpszClassName = lpszAppName;              

   if ( IS_WIN95 )
   {
      if ( !RegisterWin95( &wc ) )
         return( FALSE );
   }
   else if ( !RegisterClass( &wc ) )
      return( FALSE );

   hInst = hInstance; 

   hWnd = CreateWindow( lpszAppName, 
                        lpszTitle,    
                        WS_OVERLAPPEDWINDOW, 
                        CW_USEDEFAULT, 0, 
                        CW_USEDEFAULT, 0,  
                        NULL,              
                        NULL,              
                        hInstance,         
                        NULL               
                      );

   if ( !hWnd ) 
      return( FALSE );

   ShowWindow( hWnd, nCmdShow ); 
   UpdateWindow( hWnd );         

   while( GetMessage( &msg, NULL, 0, 0) )   
   {
      TranslateMessage( &msg ); 
      DispatchMessage( &msg );  
   }

   return( msg.wParam ); 
}


BOOL RegisterWin95( CONST WNDCLASS* lpwc )
{
   WNDCLASSEX wcex;

   wcex.style         = lpwc->style;
   wcex.lpfnWndProc   = lpwc->lpfnWndProc;
   wcex.cbClsExtra    = lpwc->cbClsExtra;
   wcex.cbWndExtra    = lpwc->cbWndExtra;
   wcex.hInstance     = lpwc->hInstance;
   wcex.hIcon         = lpwc->hIcon;
   wcex.hCursor       = lpwc->hCursor;
   wcex.hbrBackground = lpwc->hbrBackground;
   wcex.lpszMenuName  = lpwc->lpszMenuName;
   wcex.lpszClassName = lpwc->lpszClassName;

   // Added elements for Windows 95.
   //...............................
   wcex.cbSize = sizeof(WNDCLASSEX);
   wcex.hIconSm = LoadImage(wcex.hInstance, lpwc->lpszClassName, 
                            IMAGE_ICON, 16, 16,
                            LR_DEFAULTCOLOR );
			
   return RegisterClassEx( &wcex );
}


LRESULT CALLBACK WndProc( HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam )
{
static HENV  hEnv  = NULL;
static HDBC  hDBC  = NULL;
static HWND  hList = NULL;

static UCHAR szSqlStr[] = 
   "Insert into department( dept_id, dept_name, dept_head_id ) "
   "Values ( ?, ?, ? )";
static UCHAR szLBStr[256];

   switch( uMsg )
   {
      case WM_CREATE :
              {
                 RETCODE retCode;

                 // Allocate Environment and Connection.
                 //.....................................
                 SQLAllocEnv( &hEnv );
                 SQLAllocConnect( hEnv, &hDBC );

                 // Connect to the data source
                 //...........................
                 retCode = SQLConnect( hDBC,
                             "Test Data Source", SQL_NTS,
                             "DBA",              SQL_NTS,  
                             "SQL",              SQL_NTS );

                 if ( retCode != SQL_SUCCESS )
                 {
                    SQLFreeConnect( hDBC );
                    SQLFreeEnv( hEnv );
                    return( -1 );
                 }

                 if ( retCode == SQL_SUCCESS )
                 {
                    int nTabSize = 40;

                    hList = CreateWindow( "LISTBOX", "",    
                               LBS_NOTIFY | LBS_USETABSTOPS |
                               LBS_NOINTEGRALHEIGHT  | 
                               WS_BORDER  | WS_CHILD | 
                               WS_VISIBLE | WS_VSCROLL, 
                               0, 0, 0, 0, 
                               hWnd, (HMENU)101,
                               hInst, NULL );
                    
                    SendMessage( hList, LB_SETTABSTOPS, 
                                 1, (LPARAM)&nTabSize );
                 }
              }
              break;

      case WM_SIZE :
              MoveWindow( hList, 0, 0, LOWORD( lParam ), 
                                       HIWORD( lParam ), TRUE );
              break; 
              
      case WM_COMMAND :
              switch( LOWORD( wParam ) )
              {
                 case IDM_TEST :
                        {
                           HSTMT   hStmt;
                           RETCODE retCode;

                           // Set connection to Manual commit mode
                           // ..........................................
                           SQLSetConnectOption( hDBC, SQL_AUTOCOMMIT, SQL_AUTOCOMMIT_OFF );
  
                           // Allocate a statement handle for the query.
                           // ..........................................
                           SQLAllocStmt( hDBC, &hStmt );

                           // Call SQLPrepare() to compile the query.
                           // .......................................
                           retCode = SQLPrepare( hStmt, szSqlStr, SQL_NTS );
                           
                           if( retCode == SQL_SUCCESS ||
                               retCode == SQL_SUCCESS_WITH_INFO )
                           {
                              UWORD  iParams = 1;
                              SWORD  nParams;
                              SWORD  fSqlType;
                              UDWORD cbColDef; 
                              SWORD  ibScale;
                              SWORD  fNullable;

                              // Call SQLNumParams() to determine the
                              // number of parameters in the statement.
                              // ......................................
                              SQLNumParams( hStmt, &nParams );

                              sprintf( szLBStr, "Number of parameters = %d",
                                                 nParams );

                              SendMessage( hList, LB_ADDSTRING, 0, 
                                                (LPARAM)szLBStr );

                              // Call SQLDescribeParam() to determine 
                              // parameter data for each parameter 
                              // contained in the statement.
                              // ....................................
                              while( iParams <= nParams )
                              {
                                 retCode = SQLDescribeParam( hStmt, iParams, 
                                    &fSqlType, &cbColDef, 
                                    &ibScale,  &fNullable);

                                 if( retCode != SQL_SUCCESS &&
                                     retCode != SQL_SUCCESS_WITH_INFO )
                                    break;

                                 BuildOutputStr( szLBStr, fSqlType, 
                                                          cbColDef, 
                                                          ibScale,  
                                                          fNullable );

                                 SendMessage( hList, LB_ADDSTRING, 0, 
                                                   (LPARAM)szLBStr );

                                 iParams++;
                              }
                           }
                           
                           // Free the statement handle.
                           // ..........................
                           SQLFreeStmt( hStmt, SQL_DROP );
                        }
                        break;

                 case IDM_ABOUT :
                        DialogBox( hInst, "AboutBox", hWnd, (DLGPROC)About );
                        break;

                 case IDM_EXIT :
                        DestroyWindow( hWnd );
                        break;
              }
              break;
      
      case WM_DESTROY :
              PostQuitMessage(0);

              // Disconnect Environment.
              //........................
              SQLDisconnect( hDBC );

              // Free Connection and Environment.
              //.................................
              SQLFreeConnect( hDBC );
              SQLFreeEnv( hEnv );
              break;

      default :
            return( DefWindowProc( hWnd, uMsg, wParam, lParam ) );
   }

   return( 0L );
}


// Builds an output string based on the values 
// returned from the SQLDescribeParam() function.
// ..............................................

VOID BuildOutputStr( LPSTR pszLBStr, SWORD fSqlType, UDWORD cbColDef, 
                                     SWORD ibScale,  SWORD  fNullable )
{
   char szTempStr[32];

   pszLBStr[0] = '\0';

   switch( fSqlType )
   {
      case SQL_BIGINT       : strcat( pszLBStr, "BIGINT"        ); break;
      case SQL_BINARY       : strcat( pszLBStr, "BINARY"        ); break;
      case SQL_BIT          : strcat( pszLBStr, "BIT"           ); break;
      case SQL_CHAR         : strcat( pszLBStr, "CHAR"          ); break;
      case SQL_DATE         : strcat( pszLBStr, "DATE"          ); break;
      case SQL_DECIMAL      : strcat( pszLBStr, "DECIMAL"       ); break;
      case SQL_DOUBLE       : strcat( pszLBStr, "DOUBLE"        ); break;
      case SQL_FLOAT        : strcat( pszLBStr, "FLOAT"         ); break;
      case SQL_INTEGER      : strcat( pszLBStr, "INTEGER"       ); break;
      case SQL_LONGVARBINARY: strcat( pszLBStr, "LONGVARBINARY" ); break;
      case SQL_LONGVARCHAR  : strcat( pszLBStr, "LONGVARCHAR"   ); break;
      case SQL_NUMERIC      : strcat( pszLBStr, "NUMERIC"       ); break;
      case SQL_REAL         : strcat( pszLBStr, "REAL"          ); break;
      case SQL_SMALLINT     : strcat( pszLBStr, "SMALLINT"      ); break;
      case SQL_TIME         : strcat( pszLBStr, "TIME"          ); break;
      case SQL_TIMESTAMP    : strcat( pszLBStr, "TIMESTAMP"     ); break;
      case SQL_TINYINT      : strcat( pszLBStr, "TINYINT"       ); break;
      case SQL_VARBINARY    : strcat( pszLBStr, "VARBINARY"     ); break;
      case SQL_VARCHAR      : strcat( pszLBStr, "VARCHAR"       ); break;
      default:   strcat( pszLBStr, "Driver-specific data type." ); break;
   }
   strcat( pszLBStr, "\t" );

   sprintf(szTempStr, "%u", cbColDef );
   strcat( pszLBStr, szTempStr );
   strcat( pszLBStr, "\t" );

   sprintf(szTempStr, "%d", ibScale );
   itoa( ibScale, szTempStr, 10 );
   strcat( pszLBStr, szTempStr );
   strcat( pszLBStr, "\t" );

   switch( fNullable )
   {
      case SQL_NO_NULLS:         strcat( pszLBStr, "No NULLs." );    break;
      case SQL_NULLABLE:         strcat( pszLBStr, "Nullable." );    break;
      case SQL_NULLABLE_UNKNOWN: strcat( pszLBStr, "NULL unknown." );break;
   }
}


LRESULT CALLBACK About( HWND hDlg,           
                        UINT message,        
                        WPARAM wParam,       
                        LPARAM lParam)
{
   switch (message) 
   {
       case WM_INITDIALOG: 
               return (TRUE);

       case WM_COMMAND:                              
               if (   LOWORD(wParam) == IDOK         
                   || LOWORD(wParam) == IDCANCEL)    
               {
                       EndDialog(hDlg, TRUE);        
                       return (TRUE);
               }
               break;
   }

   return (FALSE); 
}
